@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 
 ## Copyright (C) 2007, 2008, 2009, 2010, 2012, 2013 Red Hat, Inc.
 ## Copyright (C) 2008 Novell, Inc.
@@ -20,8 +20,6 @@
 
 import cups
 import os
-
-import config
 from debug import *
 import debug
 
@@ -235,7 +233,6 @@ if __name__ == "__main__":
     # Demo
     from gi.repository import GObject
     set_debugging (True)
-    GObject.threads_init ()
 
     class Test:
         def __init__ (self, quit):
@@ -257,11 +254,11 @@ if __name__ == "__main__":
                                    error_handler=self.getDevices_error)
 
         def getDevices_reply (self, conn, result):
-            print conn, result
+            print (conn, result)
             self.destroy ()
 
         def getDevices_error (self, conn, exc):
-            print repr (exc)
+            print (repr (exc))
             self.destroy ()
 
     t = Test (False)
