@@ -1,6 +1,6 @@
 #!/usr/bin/python3
 
-## Copyright (C) 2007, 2008, 2009, 2010, 2012, 2013 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010, 2012, 2013, 2014 Red Hat, Inc.
 ## Copyright (C) 2008 Novell, Inc.
 ## Authors: Tim Waugh <twaugh@redhat.com>, Vincent Untz
 
@@ -208,7 +208,7 @@ class _WriteToTmpFile:
             line = tmpfile.readline ()
             while line != '':
                 os.write (fd, line.encode('UTF-8'))
-                line = tempfile.readline ()
+                line = tmpfile.readline ()
         else:
             file_object = self._kwds["file"]
             file_object.seek (0)
@@ -242,7 +242,7 @@ class PK1Connection:
             self._system_bus = None
 
         global _DevicesGet_uses_new_api
-        if _DevicesGet_uses_new_api == None and self._system_bus:
+        if _DevicesGet_uses_new_api is None and self._system_bus:
             try:
                 obj = self._system_bus.get_object(CUPS_PK_NAME, CUPS_PK_PATH)
                 proxy = dbus.Interface (obj, dbus.INTROSPECTABLE_IFACE)
@@ -324,7 +324,7 @@ class PK1Connection:
             del leftover_kwds["auth_handler"]
 
         result = [True, reply_handler, error_handler, ()]
-        if self._system_bus == None:
+        if self._system_bus is None:
             return result
 
         tup = []
@@ -461,7 +461,7 @@ class PK1Connection:
                 else:
                     device_uri = result_str[keywithaffix]
 
-            if device_uri != None:
+            if device_uri is not None:
                 devices[device_uri] = device_dict
 
             n += 1
