@@ -1,6 +1,6 @@
 ## system-config-printer
 
-## Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Red Hat, Inc.
+## Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015 Red Hat, Inc.
 ## Authors:
 ##  Florian Festi <ffesti@redhat.com>
 ##  Tim Waugh <twaugh@redhat.com>
@@ -55,7 +55,7 @@ class Printer:
         self._ppd = None # load on demand
 
     def __del__ (self):
-        if self._ppd != None:
+        if self._ppd is not None:
             os.unlink(self._ppd)
 
     def __repr__ (self):
@@ -210,7 +210,7 @@ class Printer:
                 else:
                     raise
 
-        if result == None and self._ppd != None:
+        if result is None and self._ppd is not None:
             result = cups.PPD (self._ppd)
 
         return result
@@ -368,7 +368,7 @@ class Printer:
                  attrs['job-name'] == 'Test Page')):
                 ret.append (id)
 
-                if limit != None and len (ret) == limit:
+                if limit is not None and len (ret) == limit:
                     break
         return ret
 
@@ -402,7 +402,7 @@ class Printer:
                            cups.IPP_JOB_PENDING) < cups.IPP_JOB_COMPLETED):
                 continue
             ret.append (id)
-            if limit != None and len (ret) == limit:
+            if limit is not None and len (ret) == limit:
                 break
 
         return ret
@@ -423,44 +423,43 @@ class Printer:
 
         # Also need to check system-wide lpoptions because that's how
         # previous Fedora versions set the default (bug #217395).
-        (tmpfd, tmpfname) = tempfile.mkstemp ()
-        os.remove (tmpfname)
-        try:
-            resource = "/admin/conf/lpoptions"
-            self.connection.getFile(resource, fd=tmpfd)
-        except cups.HTTPError as e:
-            (s,) = e.args
-            if s == cups.HTTP_NOT_FOUND:
-                return False
-
-            raise cups.HTTPError (s)
-
-        f = os.fdopen (tmpfd, 'r+b')
-        f.seek (0)
-        lines = [ line.decode('UTF-8') for line in f.readlines () ]
-        changed = False
-        i = 0
-        for line in lines:
-            if line.startswith ("Default "):
-                # This is the system-wide default.
-                name = line.split (' ')[1]
-                if name != self.name:
-                    # Stop it from over-riding the server default.
-                    lines[i] = "Dest " + line[8:]
-                    changed = True
-                i += 1
-
-        if changed:
-            f.seek (0)
-            f.writelines ([ line.encode('UTF-8') for line in lines ])
-            f.truncate ()
-            os.lseek (tmpfd, 0, os.SEEK_SET)
+        with tempfile.TemporaryFile () as f:
             try:
-                self.connection.putFile (resource, fd=tmpfd)
-            except cups.HTTPError:
-                return False
+                resource = "/admin/conf/lpoptions"
+                self.connection.getFile(resource, fd=f.fileno ())
+            except cups.HTTPError as e:
+                (s,) = e.args
+                if s in [cups.HTTP_NOT_FOUND, cups.HTTP_AUTHORIZATION_CANCELED]:
+                    return False
 
-        return changed
+                raise cups.HTTPError (s)
+
+            f.seek (0)
+            lines = [ line.decode('UTF-8') for line in f.readlines () ]
+            changed = False
+            i = 0
+            for line in lines:
+                if line.startswith ("Default "):
+                    # This is the system-wide default.
+                    name = line.split (' ')[1]
+                    if name != self.name:
+                        # Stop it from over-riding the server default.
+                        lines[i] = "Dest " + line[8:]
+                        changed = True
+                    i += 1
+
+            if changed:
+                f.seek (0)
+                f.writelines ([ line.encode('UTF-8') for line in lines ])
+                f.truncate ()
+                f.flush ()
+                f.seek (0)
+                try:
+                    self.connection.putFile (resource, fd=f.fileno ())
+                except cups.HTTPError:
+                    return False
+
+            return changed
 
 def getPrinters(connection):
     """
@@ -547,69 +546,69 @@ class Device:
         """
         Compare devices by order of preference.
         """
-        if other == None:
-            return 1
+        if other is None:
+            return True
 
         if self.is_class != other.is_class:
             if other.is_class:
-                return 1
-            return -1
+                return True
+            return False
         if not self.is_class and (self.type != other.type):
             # "hp"/"hpfax" before "usb" before * before "parallel" before
             # "serial"
             if other.type == "serial":
-                return 1
+                return True
             if self.type == "serial":
-                return -1
+                return False
             if other.type == "parallel":
-                return 1
+                return True
             if self.type == "parallel":
-                return -1
+                return False
             if other.type == "hp":
-                return -1
+                return False
             if self.type == "hp":
-                return 1
+                return True
             if other.type == "hpfax":
-                return -1
+                return False
             if self.type == "hpfax":
-                return 1
+                return True
             if other.type == "dnssd":
-                return -1
+                return False
             if self.type == "dnssd":
-                return 1
+                return True
             if other.type == "socket":
-                return -1
+                return False
             if self.type == "socket":
-                return 1
+                return True
             if other.type == "lpd":
-                return -1
+                return False
             if self.type == "lpd":
-                return 1
+                return True
             if other.type == "ipps":
-                return -1
+                return False
             if self.type == "ipps":
-                return 1
+                return True
             if other.type == "ipp":
-                return -1
+                return False
             if self.type == "ipp":
-                return 1
+                return True
             if other.type == "usb":
-                return -1
+                return False
             if self.type == "usb":
-                return 1
+                return True
         if self.type == "dnssd" and other.type == "dnssd":
             if other.uri.find("._pdl-datastream") != -1: # Socket
-                return -1
+                return False
             if self.uri.find("._pdl-datastream") != -1:
-                return 1
+                return True
             if other.uri.find("._printer") != -1: # LPD
-                return -1
+                return False
             if self.uri.find("._printer") != -1:
-                return 1
+                return True
             if other.uri.find("._ipp") != -1: # IPP
-                return -1
+                return False
             if self.uri.find("._ipp") != -1:
-                return 1
+                return True
         result = bool(self.id) < bool(other.id)
         if not result:
             result = self.info < other.info
@@ -666,7 +665,7 @@ def activateNewPrinter(connection, name):
     connection.acceptJobs (name)
 
     # Set as the default if there is not already a default printer.
-    if connection.getDefault () == None:
+    if connection.getDefault () is None:
         connection.setDefault (name)
 
 def copyPPDOptions(ppd1, ppd2):
