@@ -2,7 +2,7 @@
 
 ## system-config-printer
 
-## Copyright (C) 2008, 2011 Red Hat, Inc.
+## Copyright (C) 2008, 2011, 2014 Red Hat, Inc.
 ## Copyright (C) 2008 Till Kamppeter <till.kamppeter@gmail.com>
 
 ## This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@
 ## along with this program; if not, write to the Free Software
 ## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-import pycurl, urllib.request, urllib.parse, urllib.error, platform, threading, tempfile, traceback
+import requests, urllib.request, urllib.parse, urllib.error, platform, threading, tempfile, traceback
 import os, sys
 from xml.etree.ElementTree import XML
 from . import Device
@@ -46,15 +46,13 @@ class _QueryThread (threading.Thread):
         self.result = b''
 
         self.setDaemon (True)
+        _debugprint ("+%s" % self)
+
+    def __del__ (self):
+        _debugprint ("-%s" % self)
 
     def run (self):
 
-        # Callback function for pycURL collecting the data coming from
-        # the web server
-        def collect_data(result):
-            self.result += result;
-            return len(result)
-
         # CGI script to be executed
         query_command = "/query.cgi"
         # Headers for the post request
@@ -70,20 +68,16 @@ class _QueryThread (threading.Thread):
         self.result = b''
         status = 1
         try:
-            curl = pycurl.Curl()
-            curl.setopt(pycurl.SSL_VERIFYPEER, 1)
-            curl.setopt(pycurl.SSL_VERIFYHOST, 2)
-            curl.setopt(pycurl.WRITEFUNCTION, collect_data)
-            curl.setopt(pycurl.URL, self.url)
-            status = curl.perform()
-            if status == None: status = 0
-            if (status != 0):
-                self.result = sys.exc_info ()
+            req = requests.get(self.url, verify=True)
+            self.result = req.content
+            status = 0
         except:
             self.result = sys.exc_info ()
-            if status == None: status = 0
+            if status is None:
+                status = 0
 
-        if self.callback != None:
+        _debugprint ("%s: query complete" % self)
+        if self.callback is not None:
             self.callback (status, self.user_data, self.result)
 
 class OpenPrinting:
@@ -93,7 +87,7 @@ class OpenPrinting:
         locale.setlocale().
         @type language: string
         """
-        if language == None:
+        if language is None:
             import locale
             try:
                 language = locale.getlocale(locale.LC_MESSAGES)
@@ -109,6 +103,10 @@ class OpenPrinting:
         self.onlyfree = 1
         self.onlymanufacturer = 0
         _debugprint ("OpenPrinting: Init %s %s %s" % (self.language, self.onlyfree, self.onlymanufacturer))
+        _debugprint ("+%s" % self)
+
+    def __del__ (self):
+        _debugprint ("-%s" % self)
 
     def cancelOperation(self, handle):
         """
@@ -168,7 +166,7 @@ class OpenPrinting:
                     id = printer.find ("id")
                     make = printer.find ("make")
                     model = printer.find ("model")
-                    if id != None and make != None and model != None:
+                    if id is not None and make is not None and model is not None:
                         idtxt = id.text
                         maketxt = make.text
                         modeltxt = model.text
@@ -269,24 +267,37 @@ class OpenPrinting:
 
                 for driver in root.findall ('driver'):
                     id = driver.attrib.get ('id')
-                    if id == None:
+                    if id is None:
                         continue
 
                     dict = {}
                     for attribute in ['name', 'url', 'supplier', 'license',
                                       'shortdescription' ]:
                         element = driver.find (attribute)
-                        if element != None and element.text != None:
+                        if element is not None and element.text is not None:
                             dict[attribute] = _normalize_space (element.text)
 
                     element = driver.find ('licensetext')
-                    if element != None:
+                    if element is not None and element.text is not None:
                         dict['licensetext'] = element.text
+                    if not 'licensetext' in dict or \
+                       dict['licensetext'] is None:
+                        element = driver.find ('licenselink')
+                        if element is not None:
+                            license_url = element.text
+                            if license_url is not None:
+                                try:
+                                    req = requests.get(license_url, verify=True)
+                                    dict['licensetext'] = \
+                                        req.content.decode("utf-8")
+                                except:
+                                    _debugprint('Cannot retrieve %s' %
+                                                license_url)
 
                     for boolean in ['nonfreesoftware', 'recommended',
                                     'patents', 'thirdpartysupplied',
                                     'manufacturersupplied']:
-                        dict[boolean] = driver.find (boolean) != None
+                        dict[boolean] = driver.find (boolean) is not None
 
                     # Make a 'freesoftware' tag for compatibility with
                     # how the OpenPrinting API used to work (see trac
@@ -295,10 +306,10 @@ class OpenPrinting:
 
                     supportcontacts = []
                     container = driver.find ('supportcontacts')
-                    if container != None:
+                    if container is not None:
                         for sc in container.findall ('supportcontact'):
                             supportcontact = {}
-                            if sc.text != None:
+                            if sc.text is not None:
                                 supportcontact['name'] = \
                                     _normalize_space (sc.text)
                             else:
@@ -314,19 +325,19 @@ class OpenPrinting:
                         continue
 
                     container = driver.find ('functionality')
-                    if container != None:
+                    if container is not None:
                         functionality = {}
                         for attribute in ['text', 'lineart', 'graphics',
                                           'photo', 'speed']:
                             element = container.find (attribute)
-                            if element != None:
+                            if element is not None:
                                 functionality[attribute] = element.text
                         if functionality:
                             dict[container.tag] = functionality
 
                     packages = {}
                     container = driver.find ('packages')
-                    if container != None:
+                    if container is not None:
                         for arch in container.getchildren ():
                             rpms = {}
                             for package in arch.findall ('package'):
@@ -335,11 +346,11 @@ class OpenPrinting:
                                                   'release', 'url', 'pkgsys',
                                                   'fingerprint']:
                                     element = package.find (attribute)
-                                    if element != None:
+                                    if element is not None:
                                         rpm[attribute] = element.text
 
                                 repositories = package.find ('repositories')
-                                if repositories != None:
+                                if repositories is not None:
                                     for pkgsys in repositories.getchildren ():
                                         rpm.setdefault('repositories', {})[pkgsys.tag] = pkgsys.text
 
@@ -351,7 +362,7 @@ class OpenPrinting:
 
                     ppds = []
                     container = driver.find ('ppds')
-                    if container != None:
+                    if container is not None:
                         for each in container.getchildren ():
                             ppds.append (each.text)
 
