@@ -2,7 +2,7 @@
 
 ## system-config-printer
 
-## Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2014 Red Hat, Inc.
+## Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2014, 2015 Red Hat, Inc.
 ## Copyright (C) 2006 Florian Festi <ffesti@redhat.com>
 ## Copyright (C) 2006, 2007, 2008, 2009 Tim Waugh <twaugh@redhat.com>
 
@@ -72,9 +72,9 @@ _MFR_BY_RANGE = [
     ("KONICA MINOLTA", re.compile("magicolor"
                                   "|pageworks"
                                   "|pagepro")),
-    ("Kyocera Mita", re.compile("fs-"
-                                "|km-"
-                                "|taskalfa")),
+    ("Kyocera", re.compile("fs-"
+                           "|km-"
+                           "|taskalfa")),
     ("Ricoh", re.compile("aficio")),
     ("Oce", re.compile("varioprint")),
     ("Oki", re.compile("okipage|microline"))
@@ -94,7 +94,7 @@ _HP_MODEL_BY_NAME = {
 }
 
 _RE_turboprint = re.compile ("turboprint")
-_RE_version_numbers = re.compile (" v\d(?:\d*\.\d+)?(?: |$)")
+_RE_version_numbers = re.compile (" v(?:er\.)?\d(?:\d*\.\d+)?(?: |$)")
 _RE_ignore_suffix = re.compile (","
                                 "| hpijs"
                                 "| foomatic/"
@@ -168,10 +168,10 @@ def ppdMakeModelSplit (ppd_make_and_model):
         make = "Lexmark"
         model = ppd_make_and_model[22:]
     elif l.startswith ("kyocera mita "):
-        make = "Kyocera Mita"
+        make = "Kyocera"
         model = ppd_make_and_model[13:]
     elif l.startswith ("kyocera "):
-        make = "Kyocera Mita"
+        make = "Kyocera"
         model = ppd_make_and_model[8:]
     elif l.startswith ("fuji xerox "):
         make = "Fuji Xerox"
@@ -203,8 +203,9 @@ def ppdMakeModelSplit (ppd_make_and_model):
             if mfr:
                 make = mfr
 
-    # HP PostScript PPDs give NickNames like:
+    # HP and Canon PostScript PPDs give NickNames like:
     # *NickName: "HP LaserJet 4 Plus v2013.111 Postscript (recommended)"
+    # *NickName: "Canon MG4100 series Ver.3.90"
     # Find the version number and truncate at that point.  But beware,
     # other model names can legitimately look like version numbers,
     # e.g. Epson PX V500.
@@ -213,9 +214,9 @@ def ppdMakeModelSplit (ppd_make_and_model):
     modell = model.lower ()
     v = modell.find (" v")
     if v != -1:
-        # Look for " v" followed by a digit, optionally followed by more
-        # digits, a dot, and more digits; and terminated by a space of the
-        # end of the line.
+        # Look for " v" or " ver." followed by a digit, optionally
+        # followed by more digits, a dot, and more digits; and
+        # terminated by a space of the end of the line.
         vmatch = _RE_version_numbers.search (modell)
         if vmatch:
             # Found it -- truncate at that point.
@@ -460,8 +461,8 @@ class PPDs:
     def getStatusFromFit (self, fit):
         return self._fit_to_status.get (fit, xmldriverprefs.DriverType.FIT_NONE)
 
-    def orderPPDNamesByPreference (self, ppdnamelist=[],
-                                   downloadedfiles=[],
+    def orderPPDNamesByPreference (self, ppdnamelist=None,
+                                   downloadedfiles=None,
                                    make_and_model=None,
                                    devid=None, fit=None):
         """
@@ -481,6 +482,12 @@ class PPDs:
         @type fit: dict of PPD name:fit
 	@returns: string list
 	"""
+        if ppdnamelist is None:
+            ppdnamelist = []
+
+        if downloadedfiles is None:
+            downloadedfiles = []
+
         if fit == None:
             fit = {}
 
@@ -525,7 +532,7 @@ class PPDs:
         return ppdnamelist
 
     def getPPDNamesFromDeviceID (self, mfg, mdl, description="",
-                                 commandsets=[], uri=None,
+                                 commandsets=None, uri=None,
                                  make_and_model=None):
         """
 	Obtain a best-effort PPD match for an IEEE 1284 Device ID.
@@ -549,6 +556,9 @@ class PPDs:
         orig_mdl = mdl
         self._init_ids ()
 
+        if commandsets is None:
+            commandsets = []
+
         # Start with an empty result list and build it up using
         # several search methods, in increasing order of fuzziness.
         fit = {}
@@ -722,9 +732,15 @@ class PPDs:
                     fit[each] = self.FIT_EXACT_CMD
                     _debugprint (self.FIT_EXACT_CMD + ": %s" % each)
 
-            _debugprint ("Removed %s due to CMD mis-match" % failed)
-            for each in failed:
-                del fit[each]
+            if len (failed) < len ([d for (d, m) in fit.items ()
+                                    if m != 'generic']):
+                _debugprint ("Removed %s due to CMD mis-match" % failed)
+                for each in failed:
+                    del fit[each]
+            else:
+                _debugprint ("Not removing %s " % failed +
+                             "due to CMD mis-match as it would "
+                             "leave nothing good")
 
         if not fit:
             fallbacks = ["textonly.ppd", "postscript.ppd"]
@@ -767,8 +783,8 @@ class PPDs:
         return fit
 
     def getPPDNameFromDeviceID (self, mfg, mdl, description="",
-                                commandsets=[], uri=None,
-                                downloadedfiles=[],
+                                commandsets=None, uri=None,
+                                downloadedfiles=None,
                                 make_and_model=None):
         """
 	Obtain a best-effort PPD match for an IEEE 1284 Device ID.
@@ -804,6 +820,12 @@ class PPDs:
 	@returns: an integer,string pair of (status,ppd-name)
 	"""
 
+        if commandsets is None:
+            commandsets = []
+
+        if downloadedfiles is None:
+            downloadedfiles = []
+
         fit = self.getPPDNamesFromDeviceID (mfg, mdl, description,
                                             commandsets, uri,
                                             make_and_model)
@@ -946,9 +968,13 @@ class PPDs:
 
         return (fit, ppdnamelist)
 
-    def _getPPDNameFromCommandSet (self, commandsets=[]):
+    def _getPPDNameFromCommandSet (self, commandsets=None):
         """Return ppd-name list or None, given a list of strings representing
         the command sets supported."""
+
+        if commandsets is None:
+            commandsets = []
+
         try:
             self._init_makes ()
             models = self.makes["Generic"]
@@ -1125,190 +1151,3 @@ class PPDs:
 
 def _show_help():
     print ("usage: ppds.py [--deviceid] [--list-models] [--list-ids] [--debug]")
-
-def _self_test(argv):
-    import sys, getopt
-    try:
-        opts, args = getopt.gnu_getopt (argv[1:], '',
-                                        ['help',
-                                         'deviceid',
-                                         'list-models',
-                                         'list-ids',
-                                         'debug'])
-    except getopt.GetoptError:
-        _show_help()
-        sys.exit (1)
-
-    stdin_deviceid = False
-    list_models = False
-    list_ids = False
-
-    for opt, optarg in opts:
-        if opt == "--help":
-            _show_help ()
-            sys.exit (0)
-        if opt == "--deviceid":
-            stdin_deviceid = True
-        elif opt == "--list-models":
-            list_models = True
-        elif opt == "--list-ids":
-            list_ids = True
-        elif opt == "--debug":
-            def _dprint(x):
-                try:
-                    print (x)
-                except:
-                    pass
-
-            set_debugprint_fn (_dprint)
-
-    picklefile="pickled-ppds"
-    import pickle
-    try:
-        with open (picklefile, "rb") as f:
-            cupsppds = pickle.load (f)
-    except IOError:
-        with open (picklefile, "wb") as f:
-            c = cups.Connection ()
-            try:
-                cupsppds = c.getPPDs2 ()
-                print ("Using getPPDs2()")
-            except AttributeError:
-                # Need pycups >= 1.9.52 for getPPDs2
-                cupsppds = c.getPPDs ()
-                print ("Using getPPDs()")
-
-            pickle.dump (cupsppds, f)
-
-    xml_dir = os.environ.get ("top_srcdir")
-    if xml_dir:
-        xml_dir = os.path.join (xml_dir, "xml")
-
-    ppds = PPDs (cupsppds, xml_dir=xml_dir)
-    makes = ppds.getMakes ()
-    models_count = 0
-    for make in makes:
-        models = ppds.getModels (make)
-        models_count += len (models)
-        if list_models:
-            print (make)
-            for model in models:
-                print ("  " + model)
-    print ("%d makes, %d models" % (len (makes), models_count))
-    ppds.getPPDNameFromDeviceID ("HP", "PSC 2200 Series")
-    makes = list(ppds.ids.keys ())
-    models_count = 0
-    for make in makes:
-        models = ppds.ids[make]
-        models_count += len (models)
-        if list_ids:
-            print (make)
-            for model in models:
-                print ("  %s (%d)" % (model, len (ppds.ids[make][model])))
-                for driver in ppds.ids[make][model]:
-                    print ("    " + driver)
-    print ("%d ID makes, %d ID models" % (len (makes), models_count))
-
-    print ("\nID matching tests\n")
-
-    MASK_STATUS = (1 << 2) - 1
-    FLAG_INVERT = (1 << 2)
-    FLAG_IGNORE_STATUS = (1 << 3)
-    idlist = [
-        # Format is:
-        # (ID string, max status code (plus flags),
-        #  expected ppd-make-and-model RE match)
-
-        # Specific models
-        ("MFG:EPSON;CMD:ESCPL2,BDC,D4,D4PX;MDL:Stylus D78;CLS:PRINTER;"
-         "DES:EPSON Stylus D78;", 1, 'Epson Stylus D68'),
-        ("MFG:Hewlett-Packard;MDL:LaserJet 1200 Series;"
-         "CMD:MLC,PCL,POSTSCRIPT;CLS:PRINTER;", 0, 'HP LaserJet 1200'),
-        ("MFG:Hewlett-Packard;MDL:LaserJet 3390 Series;"
-         "CMD:MLC,PCL,POSTSCRIPT;CLS:PRINTER;", 0, 'HP LaserJet 3390'),
-        ("MFG:Hewlett-Packard;MDL:PSC 2200 Series;CMD:MLC,PCL,PML,DW-PCL,DYN;"
-         "CLS:PRINTER;1284.4DL:4d,4e,1;", 0, "HP PSC 22[01]0"),
-        ("MFG:HEWLETT-PACKARD;MDL:DESKJET 990C;CMD:MLC,PCL,PML;CLS:PRINTER;"
-         "DES:Hewlett-Packard DeskJet 990C;", 0, "HP DeskJet 990C"),
-        ("CLASS:PRINTER;MODEL:HP LaserJet 6MP;MANUFACTURER:Hewlett-Packard;"
-         "DESCRIPTION:Hewlett-Packard LaserJet 6MP Printer;"
-         "COMMAND SET:PJL,MLC,PCLXL,PCL,POSTSCRIPT;", 0, "HP LaserJet 6MP"),
-        # Canon PIXMA iP3000 (from gutenprint)
-        ("MFG:Canon;CMD:BJL,BJRaster3,BSCCe;SOJ:TXT01;MDL:iP3000;CLS:PRINTER;"
-         "DES:Canon iP3000;VER:1.09;STA:10;FSI:03;", 1, "Canon PIXMA iP3000"),
-        ("MFG:HP;MDL:Deskjet 5400 series;CMD:MLC,PCL,PML,DW-PCL,DESKJET,DYN;"
-         "1284.4DL:4d,4e,1;CLS:PRINTER;DES:5440;",
-         1, "HP DeskJet (5440|5550)"), # foomatic-db-hpijs used to say 5440
-        ("MFG:Hewlett-Packard;MDL:HP LaserJet 3390;"
-         "CMD:PJL,MLC,PCL,POSTSCRIPT,PCLXL;",
-         0, "HP LaserJet 3390"),
-        # Ricoh printers should use PostScript versions of
-        # manufacturer's PPDs (bug #550315 comment #8).
-        ("MFG:RICOH;MDL:Aficio 3045;",
-         0, "Ricoh Aficio 3045 PS"),
-        # Don't mind which driver gets used here so long as it isn't
-        # gutenprint (bug #645993).
-        ("MFG:Brother;MDL:HL-2030;",
-         0 | FLAG_INVERT | FLAG_IGNORE_STATUS, ".*Gutenprint"),
-        # Make sure we get a colour driver for this one, see launchpad
-        # #669152.
-        ("MFG:Xerox;MDL:6250DP;",
-         1, ".*(Postscript|pcl5e)"),
-
-        # Generic models
-        ("MFG:New;MDL:Unknown PS Printer;CMD:POSTSCRIPT;",
-         2, "Generic postscript printer"),
-        # Make sure pxlcolor is used for PCLXL.  The gutenprint driver
-        # is black and white, and pxlcolor is the foomatic-recommended
-        # generic driver for "Generic PCL 6/PCL XL Printer".
-        ("MFG:New;MDL:Unknown PCL6 Printer;CMD:PCLXL;", 2,
-         "Generic PCL 6.*pxlcolor"),
-        ("MFG:New;MDL:Unknown PCL5e Printer;CMD:PCL5e;", 2, "Generic PCL 5e"),
-        ("MFG:New;MDL:Unknown PCL5c Printer;CMD:PCL5c;", 2, "Generic PCL 5c"),
-        ("MFG:New;MDL:Unknown PCL5 Printer;CMD:PCL5;", 2, "Generic PCL 5"),
-        ("MFG:New;MDL:Unknown PCL3 Printer;CMD:PCL;", 2, "Generic PCL"),
-        ("MFG:New;MDL:Unknown Printer;", 100, None),
-        ]
-
-    if stdin_deviceid:
-        idlist = [(input ('Device ID: '), 2, '')]
-
-    all_passed = True
-    for id, max_status_code, modelre in idlist:
-        flags = max_status_code & ~MASK_STATUS
-        max_status_code &= MASK_STATUS
-        id_dict = parseDeviceID (id)
-        (status, ppdname) = ppds.getPPDNameFromDeviceID (id_dict["MFG"],
-                                                         id_dict["MDL"],
-                                                         id_dict["DES"],
-                                                         id_dict["CMD"])
-        ppddict = ppds.getInfoFromPPDName (ppdname)
-        if flags & FLAG_IGNORE_STATUS:
-            status = max_status_code
-
-        if status < max_status_code:
-            success = True
-        else:
-            if status == max_status_code:
-                match = re.match (modelre,
-                                  _singleton (ppddict['ppd-make-and-model']),
-                                  re.I)
-                success = match != None
-            else:
-                success = False
-
-
-        if flags & FLAG_INVERT:
-            success = not success
-
-        if success:
-            result = "PASS"
-        else:
-            result = "*** FAIL ***"
-
-        print ("%s: %s %s (%s)" % (result, id_dict["MFG"], id_dict["MDL"],
-                                  _singleton (ppddict['ppd-make-and-model'])))
-        all_passed = all_passed and success
-
-    if not all_passed:
-        raise RuntimeError
