@@ -1,5 +1,5 @@
 
-## Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015 Red Hat, Inc.
 ## Authors:
 ##  Tim Waugh <twaugh@redhat.com>
 ##  Jiri Popelka <jpopelka@redhat.com>
@@ -24,6 +24,9 @@ import cups
 import dbus
 import dbus.glib
 import dbus.service
+import threading
+import gi
+gi.require_version('Notify', '0.7')
 from gi.repository import Notify
 from gi.repository import GLib
 from gi.repository import GObject
@@ -51,6 +54,7 @@ from functools import reduce
 cups.require("1.9.47")
 
 try:
+    gi.require_version('GnomeKeyring', '1.0')
     from gi.repository import GnomeKeyring
     USE_KEYRING=True
 except ImportError:
@@ -70,8 +74,11 @@ SEARCHING_ICON="document-print-preview"
 Notify.init('System Config Printer Notification')
 
 class PrinterURIIndex:
-    def __init__ (self, names=[]):
+    def __init__ (self, names=None):
         self.printer = {}
+        if names is None:
+            names = []
+
         self.names = names
         self._collect_names ()
 
@@ -140,11 +147,11 @@ class PrinterURIIndex:
 
     def _map_printer (self, uri=None, name=None, connection=None):
         try:
-            if connection == None:
+            if connection is None:
                 connection = cups.Connection ()
 
             r = ['printer-name', 'printer-uri-supported', 'printer-more-info']
-            if uri != None:
+            if uri is not None:
                 attrs = connection.getPrinterAttributes (uri=uri,
                                                          requested_attributes=r)
             else:
@@ -159,18 +166,18 @@ class PrinterURIIndex:
 
         name = attrs['printer-name']
         self.update_from_attrs (name, attrs)
-        if uri != None:
+        if uri is not None:
             self.printer[uri] = name
         return name
 
 
 class CancelJobsOperation(GObject.GObject):
     __gsignals__ = {
-        'destroy':     (GObject.SIGNAL_RUN_LAST, None, ()),
-        'job-deleted': (GObject.SIGNAL_RUN_LAST, None, (int,)),
-        'ipp-error':   (GObject.SIGNAL_RUN_LAST, None,
+        'destroy':     (GObject.SignalFlags.RUN_LAST, None, ()),
+        'job-deleted': (GObject.SignalFlags.RUN_LAST, None, (int,)),
+        'ipp-error':   (GObject.SignalFlags.RUN_LAST, None,
                         (int, GObject.TYPE_PYOBJECT)),
-        'finished':    (GObject.SIGNAL_RUN_LAST, None, ())
+        'finished':    (GObject.SignalFlags.RUN_LAST, None, ())
         }
 
     def __init__ (self, parent, host, port, encryption, jobids, purge_job):
@@ -314,10 +321,11 @@ class JobViewer (GtkGUI):
                                    'job-printer-uri',
                                    'job-state',
                                    'time-at-creation',
+                                   'auth-info-required',
                                    'job-preserved'])
 
     __gsignals__ = {
-        'finished':    (GObject.SIGNAL_RUN_LAST, None, ())
+        'finished':    (GObject.SignalFlags.RUN_LAST, None, ())
         }
 
     def __init__(self, bus=None, loop=None,
@@ -543,7 +551,7 @@ class JobViewer (GtkGUI):
             self.statusicon.set_visible (False)
 
         # D-Bus
-        if bus == None:
+        if bus is None:
             bus = dbus.SystemBus ()
 
         self.connect_signals ()
@@ -552,6 +560,7 @@ class JobViewer (GtkGUI):
         self.port = cups.getPort ()
         self.encryption = cups.getEncryption ()
         self.monitor = monitor.Monitor (bus=bus, my_jobs=my_jobs,
+                                        specific_dests=specific_dests,
                                         host=self.host, port=self.port,
                                         encryption=self.encryption)
         self.monitor.connect ('refresh', self.on_refresh)
@@ -634,7 +643,7 @@ class JobViewer (GtkGUI):
                         pass
                     notification.closed = True
 
-        if self.job_creation_times_timer != None:
+        if self.job_creation_times_timer is not None:
             GLib.source_remove (self.job_creation_times_timer)
             self.job_creation_times_timer = None
 
@@ -688,7 +697,7 @@ class JobViewer (GtkGUI):
 
                 if loc:
                     w.set_skip_taskbar_hint (True)
-                    if aw != None:
+                    if aw is not None:
                         aw.set_skip_taskbar_hint (True)
                     self.JobsWindow.iconify ()
                 else:
@@ -697,7 +706,7 @@ class JobViewer (GtkGUI):
                 self.JobsWindow.present ()
                 self.JobsWindow.set_skip_taskbar_hint (False)
                 aw = self.JobsAttributesWindow.get_window()
-                if aw != None:
+                if aw is not None:
                     aw.set_skip_taskbar_hint (False)
 
         self.JobsWindow.visible = not visible
@@ -798,7 +807,7 @@ class JobViewer (GtkGUI):
         self.jobiters[job] = iter
 
         range = self.treeview.get_visible_range ()
-        if range != None:
+        if range is not None:
             (start, end) = range
             if (self.store.get_sort_column_id () == (0,
                                                      Gtk.SortType.DESCENDING) and
@@ -833,7 +842,7 @@ class JobViewer (GtkGUI):
         if r:
             attrs = None
             try:
-                if connection == None:
+                if connection is None:
                     connection = cups.Connection (host=self.host,
                                                   port=self.port,
                                                   encryption=self.encryption)
@@ -861,8 +870,13 @@ class JobViewer (GtkGUI):
             s = int (jstate)
 
             if s in [cups.IPP_JOB_HELD, cups.IPP_JOB_STOPPED]:
-                jattrs = ['job-state', 'job-hold-until']
+                jattrs = ['job-state', 'job-hold-until', 'job-printer-uri']
                 pattrs = ['auth-info-required', 'device-uri']
+                # The current job-printer-uri may differ from the one that
+                # is returned when we request it over the connection.
+                # So while we use it to query the printer attributes we
+                # Update it afterwards to make sure that we really
+                # have the one cups uses in the job attributes.
                 uri = data.get ('job-printer-uri')
                 c = authconn.Connection (self.JobsWindow,
                                          host=self.host,
@@ -915,6 +929,8 @@ class JobViewer (GtkGUI):
                                      data.get ('job-printer-uri'),
                                      data.get ('auth-info-required', []),
                                      self.applet)
+
+        self.submenu_set = False
         self.update_sensitivity ()
 
     def get_authentication (self, job, device_uri, printer_uri,
@@ -941,12 +957,12 @@ class JobViewer (GtkGUI):
                     informational_attrs["domain"] = str (group)
                 else:
                     (serverport, rest) = urllib.parse.splithost (rest)
-                    if serverport == None:
+                    if serverport is None:
                         server = None
                     else:
                         (server, port) = urllib.parse.splitnport (serverport)
 
-                if scheme == None or server == None:
+                if scheme is None or server is None:
                     try_keyring = False
                 else:
                     informational_attrs.update ({ "server": str (server.lower ()),
@@ -1006,7 +1022,7 @@ class JobViewer (GtkGUI):
                 except RuntimeError:
                     try_keyring = False
 
-            if try_keyring and auth_info != None:
+            if try_keyring and auth_info is not None:
                 try:
                     c._begin_operation (_("authenticating job"))
                     c.authenticateJob (job, auth_info)
@@ -1114,7 +1130,7 @@ class JobViewer (GtkGUI):
                 auth_info_required = getattr (dialog,
                                               "auth_info_required",
                                               None)
-                if keyring_attrs != None and auth_info_required != None:
+                if keyring_attrs is not None and auth_info_required is not None:
                     try:
                         ind = auth_info_required.index ('username')
                         keyring_attrs['user'] = auth_info[ind]
@@ -1262,9 +1278,11 @@ class JobViewer (GtkGUI):
                 except KeyError:
                     uri = None
                 menuitem = Gtk.MenuItem (label=printer)
-                menuitem.set_sensitive (uri != None)
+                menuitem.set_sensitive (uri is not None)
                 menuitem.show ()
-                menuitem.connect ('activate', self.on_job_move_activate, uri)
+                self._submenu_connect_hack (menuitem,
+                                            self.on_job_move_activate,
+                                            uri)
                 printers_menu.append (menuitem)
 
             self.move_job_menuitem.set_submenu (printers_menu)
@@ -1307,7 +1325,7 @@ class JobViewer (GtkGUI):
 
     def poll_subprocess(self, process):
         returncode = process.poll ()
-        return returncode == None
+        return returncode is None
 
     def on_icon_quit_activate (self, menuitem):
         self.cleanup ()
@@ -1442,7 +1460,7 @@ class JobViewer (GtkGUI):
                     format = document.get('document-format', '')
 
                     # if there's no document-name retrieved
-                    if name == None:
+                    if name is None:
                         # give the default filename some meaningful name
                         name = _("retrieved")+str(document_number)
                         # add extension according to format
@@ -1453,7 +1471,7 @@ class JobViewer (GtkGUI):
                         elif format.find('application/') != -1:
                             name = name + format.replace('application/', '.')
 
-                    if tempfile != None:
+                    if tempfile is not None:
                         dialog = Gtk.FileChooserDialog (title=_("Save File"),
                                             transient_for=self.JobsWindow,
                                             action=Gtk.FileChooserAction.SAVE)
@@ -1487,6 +1505,16 @@ class JobViewer (GtkGUI):
         del c
         self.update_monitor ()
 
+    def _submenu_connect_hack (self, item, callback, *args):
+        # See https://bugzilla.gnome.org/show_bug.cgi?id=695488
+        only_once = threading.Semaphore (1)
+        def handle_event (item, event=None):
+            if only_once.acquire (False):
+                GObject.idle_add (callback, item, *args)
+
+        return (item.connect ('button-press-event', handle_event),
+                item.connect ('activate', handle_event))
+
     def on_job_move_activate(self, menuitem, job_printer_uri):
         try:
             c = authconn.Connection (self.JobsWindow,
@@ -1575,7 +1603,7 @@ class JobViewer (GtkGUI):
 
     def update_job_attributes_viewer(self, jobid, conn=None):
         """ Update attributes store with new values. """
-        if conn != None:
+        if conn is not None:
             c = conn
         else:
             try:
@@ -1614,7 +1642,7 @@ class JobViewer (GtkGUI):
     ## Icon manipulation
     def add_state_reason_emblem (self, pixbuf, printer=None):
         worst_reason = None
-        if printer == None and self.worst_reason != None:
+        if printer is None and self.worst_reason is not None:
             # Check that it's valid.
             printer = self.worst_reason.get_printer ()
             found = False
@@ -1622,17 +1650,17 @@ class JobViewer (GtkGUI):
                 if reason == self.worst_reason:
                     worst_reason = self.worst_reason
                     break
-            if worst_reason == None:
+            if worst_reason is None:
                 self.worst_reason = None
 
-        if printer != None:
+        if printer is not None:
             for reason in self.printer_state_reasons.get (printer, []):
-                if worst_reason == None:
+                if worst_reason is None:
                     worst_reason = reason
                 elif reason > worst_reason:
                     worst_reason = reason
 
-        if worst_reason != None:
+        if worst_reason is not None:
             level = worst_reason.get_level ()
             if level > StateReason.REPORT:
                 # Add an emblem to the icon.
@@ -1659,7 +1687,7 @@ class JobViewer (GtkGUI):
         if not self.applet:
             return
 
-        if have_jobs == None:
+        if have_jobs is None:
             have_jobs = len (self.jobs.keys ()) > 0
 
         if have_jobs:
@@ -1683,7 +1711,7 @@ class JobViewer (GtkGUI):
         if not self.applet:
             return
 
-        if tooltip == None:
+        if tooltip is None:
             num_jobs = len (self.jobs)
             if num_jobs == 0:
                 tooltip = _("No documents queued")
@@ -1728,7 +1756,7 @@ class JobViewer (GtkGUI):
 
         Gdk.threads_enter ()
         self.statusbar.pop (0)
-        if self.worst_reason != None:
+        if self.worst_reason is not None:
             (title, tooltip) = self.worst_reason.get_description ()
             self.statusbar.push (0, tooltip)
         else:
@@ -1832,7 +1860,7 @@ class JobViewer (GtkGUI):
         job = self.jobs.get (jobid, {})
         document = job.get ('job-name', _("Unknown"))
         printer_uri = job.get ('job-printer-uri')
-        if printer_uri != None:
+        if printer_uri is not None:
             # Determine if this printer is remote.  There's no need to
             # show a notification if the printer is connected to this
             # machine.
@@ -1841,7 +1869,7 @@ class JobViewer (GtkGUI):
             # determined this if authentication was required.
             device_uri = job.get ('device-uri')
 
-            if device_uri == None:
+            if device_uri is None:
                 pattrs = ['device-uri']
                 c = authconn.Connection (self.JobsWindow,
                                          host=self.host,
@@ -1855,7 +1883,7 @@ class JobViewer (GtkGUI):
 
                 device_uri = attrs.get ('device-uri')
 
-            if device_uri != None:
+            if device_uri is not None:
                 (scheme, rest) = urllib.parse.splittype (device_uri)
                 if scheme not in ['socket', 'ipp', 'http', 'smb']:
                     return
@@ -2200,7 +2228,7 @@ class JobViewer (GtkGUI):
         except KeyError:
             debugprint ("Couldn't find state reason (no reasons)!")
 
-        if reason != None:
+        if reason is not None:
             tuple = reason.get_tuple ()
         else:
             debugprint ("Couldn't find state reason in list!")
@@ -2213,7 +2241,7 @@ class JobViewer (GtkGUI):
                     tuple = (level, p, r)
                     break
 
-            if tuple == None:
+            if tuple is None:
                 debugprint ("Unexpected now_connected signal "
                             "(reason not in notifications list)")
                 return
@@ -2306,7 +2334,7 @@ class JobViewer (GtkGUI):
         elif s == cups.IPP_JOB_HELD:
             state = _("Held")
             until = data.get ('job-hold-until')
-            if until != None:
+            if until is not None:
                 try:
                     colon1 = until.find (':')
                     if colon1 != -1:
@@ -2334,7 +2362,7 @@ class JobViewer (GtkGUI):
                         os.environ["TZ"] = "UTC"
                         simpletime = time.mktime (hold)
 
-                        if old_tz == None:
+                        if old_tz is None:
                             del os.environ["TZ"]
                         else:
                             os.environ["TZ"] = old_tz
@@ -2364,10 +2392,10 @@ class JobViewer (GtkGUI):
                           cups.IPP_JOB_CANCELED: _("Canceled"),
                           cups.IPP_JOB_ABORTED: _("Aborted"),
                           cups.IPP_JOB_COMPLETED: _("Completed") }[s]
-            except IndexError:
+            except KeyError:
                 pass
 
-        if state == None:
+        if state is None:
             state = _("Unknown")
 
         return state
@@ -2397,7 +2425,7 @@ class JobViewer (GtkGUI):
                                   copy.get_width () / 2 - 1,
                                   copy.get_height () / 2 - 1,
                                   1.0, 1.0,
-                                  GdkPixbuf.InterpType.NEAREST, 255)
+                                  GdkPixbuf.InterpType.BILINEAR, 255)
                 icon = copy
             except GObject.GError:
                 debugprint ("No %s icon available" % Gtk.STOCK_MEDIA_PAUSE)
