@@ -34,6 +34,7 @@ import cups
 import cupshelpers
 import dnssdresolve
 import jobviewer
+import killtimer
 import newprinter
 import PhysicalDevice
 import ppdcache
@@ -51,42 +52,6 @@ CONFIG_JOBVIEWER_IFACE=CONFIG_IFACE + ".JobViewer"
 g_ppds = None
 g_killtimer = None
 
-class KillTimer:
-    def __init__ (self, timeout=30, killfunc=None):
-        self._timeout = timeout
-        self._killfunc = killfunc
-        self._holds = 0
-        self._add_timeout ()
-
-    def _add_timeout (self):
-        self._timer = GLib.timeout_add_seconds (self._timeout, self._kill)
-
-    def _kill (self):
-        debugprint ("Timeout (%ds), exiting" % self._timeout)
-        if self._killfunc:
-            self._killfunc ()
-        else:
-            sys.exit (0)
-
-    def add_hold (self):
-        if self._holds == 0:
-            debugprint ("Kill timer stopped")
-            GLib.source_remove (self._timer)
-
-        self._holds += 1
-
-    def remove_hold (self):
-        if self._holds > 0:
-            self._holds -= 1
-            if self._holds == 0:
-                debugprint ("Kill timer started")
-                self._add_timeout ()
-
-    def alive (self):
-        if self._holds == 0:
-            GLib.source_remove (self._timer)
-            self._add_timeout ()
-
 class FetchedPPDs(GObject.GObject):
     __gsignals__ = {
         'ready': (GObject.SIGNAL_RUN_LAST, None, ()),
@@ -108,6 +73,7 @@ class FetchedPPDs(GObject.GObject):
 
     def run (self):
         debugprint ("FetchPPDs: running")
+        self._ppds = None
         self._cupsconn.getPPDs2 (reply_handler=self._cups_getppds_reply,
                                  error_handler=self._cups_error)
 
@@ -166,6 +132,10 @@ class GetBestDriversRequest:
         self.error_handler (exc)
 
     def _ppds_ready (self, fetchedppds):
+        if not fetchedppds.is_ready ():
+            # PPDs being reloaded. Wait for next 'ready' signal.
+            return
+
         self._disconnect_signals ()
         ppds = fetchedppds.get_ppds ()
 
@@ -195,6 +165,33 @@ class GetBestDriversRequest:
                                                           fit=fit)
             ppdname = ppdnamelist[0]
             status = fit[ppdname]
+
+            try:
+                if status != "exact" and not self.download_tried:
+                    self.download_tried = True
+                    self.dialog = newprinter.NewPrinterGUI()
+                    self.dialog.NewPrinterWindow.set_modal (False)
+                    self.handles = \
+                                   [self.dialog.connect ('dialog-canceled',
+                                                         self.on_dialog_canceled),
+                                    self.dialog.connect ('driver-download-checked',
+                                                         self.on_driver_download_checked)]
+
+                    self.reply_if_fail = [(x, fit[x]) for x in ppdnamelist]
+                    if not self.dialog.init ('download_driver',
+                                             devid=self.device_id):
+                        try:
+                            g_killtimer.remove_hold ()
+                        finally:
+                            e = RuntimeError ("Failed to launch dialog")
+                            self.error_handler (e)
+
+                    return
+            except:
+                # Ignore driver download if packages needed for the GUI are not
+                # installed or if no windows can be opened
+                pass
+
             g_killtimer.remove_hold ()
             self.reply_handler ([(x, fit[x]) for x in ppdnamelist])
         except Exception as e:
@@ -205,13 +202,29 @@ class GetBestDriversRequest:
 
             self.error_handler (e)
 
+    def _destroy_dialog (self):
+        for handle in self.handles:
+            self.dialog.disconnect (handle)
+
+        self.dialog.destroy ()
+        del self.dialog
+
     def on_driver_download_checked(self, obj, installed_files):
-        self.installed_files = installed_files
-        self.loop.quit ()
+        if len (installed_files) > 0:
+            debugprint ("GetBestDrivers request: Re-fetch PPDs after driver download")
+            self._signals.append (g_ppds.connect ('ready', self._ppds_ready))
+            self._signals.append (g_ppds.connect ('error', self._ppds_error))
+            g_ppds.run ()
+            return
+
+        g_killtimer.remove_hold ()
+        self._destroy_dialog ()
+        self.reply_handler (self.reply_if_fail)
 
     def on_dialog_canceled(self, obj):
-        self.installed_files = []
-        self.loop.quit ()
+        g_killtimer.remove_hold ()
+        self._destroy_dialog ()
+        self.reply_handler (self.reply_if_fail)
 
 class GroupPhysicalDevicesRequest:
     def __init__ (self, devices, reply_handler, error_handler):
@@ -247,7 +260,7 @@ class GroupPhysicalDevicesRequest:
         # (in self.devices) have been modified.
         try:
             self.physdevs = []
-            for device_uri, deviceobj in self.deviceobjs.iteritems ():
+            for device_uri, deviceobj in self.deviceobjs.items ():
                 newphysicaldevice = PhysicalDevice.PhysicalDevice (deviceobj)
                 matched = False
                 try:
@@ -285,6 +298,7 @@ class ConfigPrintingNewPrinterDialog(dbus.service.Object):
         debugprint ("+%s" % self)
 
     def __del__ (self):
+        self.dialog.destroy ()
         debugprint ("-%s" % self)
 
     @dbus.service.method(dbus_interface=CONFIG_NEWPRINTERDIALOG_IFACE,
@@ -344,24 +358,28 @@ class ConfigPrintingNewPrinterDialog(dbus.service.Object):
         pass
 
     def on_dialog_canceled(self, obj):
+        debugprint ("%s: dialog canceled" % self)
         g_killtimer.remove_hold ()
         self.DialogCanceled ()
         self.remove_handles ()
         self.remove_from_connection ()
 
     def on_printer_added(self, obj, name):
+        debugprint ("%s: printer added" % self)
         g_killtimer.remove_hold ()
         self.PrinterAdded (name)
         self.remove_handles ()
         self.remove_from_connection ()
 
     def on_printer_modified(self, obj, name, ppd_has_changed):
+        debugprint ("%s: printer modified" % self)
         g_killtimer.remove_hold ()
         self.PrinterModifed (name, ppd_has_changed)
         self.remove_handles ()
         self.remove_from_connection ()
 
     def on_driver_download_checked(self, obj, installed_files):
+        debugprint ("%s: driver download checked" % self)
         g_killtimer.remove_hold ()
         self.DriverDownloadChecked (installed_files)
         self.remove_handles ()
@@ -562,6 +580,7 @@ if __name__ == '__main__':
         for opt in sys.argv[1:]:
             if opt == "--debug":
                 set_debugging (True)
+                cupshelpers.set_debugprint_fn (debugprint)
             elif opt == "--client":
                 client_demo = True
 
@@ -570,7 +589,7 @@ if __name__ == '__main__':
         sys.exit (0)
 
     debugprint ("Service running...")
-    g_killtimer = KillTimer (killfunc=Gtk.main_quit)
+    g_killtimer = killtimer.KillTimer (killfunc=Gtk.main_quit)
     cp = ConfigPrinting ()
     Gdk.threads_enter ()
     Gtk.main ()
