@@ -1,6 +1,6 @@
 #!/usr/bin/python3
 
-## Copyright (C) 2007, 2008, 2009, 2010, 2012, 2013 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010, 2012, 2013, 2014 Red Hat, Inc.
 ## Authors:
 ##  Tim Waugh <twaugh@redhat.com>
 ##  Jiri Popelka <jpopelka@redhat.com>
@@ -52,11 +52,24 @@ class StateReason:
         return self.printer
 
     def get_level (self):
-        if self.level != None:
+        if self.level is not None:
             return self.level
 
         if (self.reason.endswith ("-report") or
-            self.reason == "connecting-to-device"):
+            self.reason in ["connecting-to-device",
+                            "cups-ipp-missing-cancel-job",
+                            "cups-ipp-missing-get-job-attributes",
+                            "cups-ipp-missing-get-printer-attributes",
+                            "cups-ipp-missing-job-history",
+                            "cups-ipp-missing-job-id",
+                            "cups-ipp-missing-job-state",
+                            "cups-ipp-missing-operations-supported",
+                            "cups-ipp-missing-print-job",
+                            "cups-ipp-missing-printer-is-accepting-jobs",
+                            "cups-ipp-missing-printer-state-reasons",
+                            "cups-ipp-missing-send-document",
+                            "cups-ipp-missing-validate-job",
+                            "cups-ipp-wrong-http-version"]):
             self.level = self.REPORT
         elif self.reason.endswith ("-warning"):
             self.level = self.WARNING
@@ -143,7 +156,7 @@ class StateReason:
                     for scheme in schemes:
                         lreason = self._ppd.localizeIPPReason(self.reason,
                                                               scheme)
-                        if lreason != None:
+                        if lreason is not None:
                             localized_reason = localized_reason + lreason + ", "
                     if localized_reason != "":
                         reason = localized_reason[:-2]
